%--------------------------------------------------------------------------
% function [map_out,zc] = FT_prepare_phase_map_for_finesse(map,curvature)
%
% A Matlab function which prepares a (phase) mirror map for use in Finesse,
% using Zernike polynomials to remove effects such as offset, tilt and 
% residual curvature.  The function then saves the resulting map in the
% file format for use in Finesse, as well as creating and saving a map
% representing the aperture created by the mirror.
%
% map:          Input map to be prepared and converted for Finesse
% curvature:    Variable which determines what curvature to remove from the
%               map.
%               0:  remove residual curvature (Z20 term)
%               1:  remove residual and astigmatic curvature (Z20 and Z22
%                   terms)
%               2:  no curvature removal
%
% map_out:      Returned map, after preparation for Finesse
% zc:           Zernike content.  Summary of the zernike polynomials in the
%               original map.
% 
% Part of the Simtools package, http://www.gwoptics.org/simtools
% Charlotte Bond    04.05.2012
%--------------------------------------------------------------------------
%

function [map_out,zc] = FT_prepare_phase_map_for_finesse(map,curvature)
    
    % TO DO: 04.05.2012
    % Add in gaussian weighting function

    baseid = 'prepare_phase_map_for_finesse';
    
    % Error messages
    if (curvature<0 || curvature >2 || round(curvature)~=curvature)
        msg = 'Invalid value: curvature must be 0, 1 or 2'
        msgid = [baseid,':checkarguments']
        error(msgid,result);
    end

    % Output filenames
    basename = map.name;
    f_mapname = sprintf('%s',basename,'_for_finesse.txt');
    a_mapname = sprintf('%s',basename,'_aperture.txt');
    zc_name = sprintf('%s',basename,'_zernike_content');
    results_name = sprintf('%s',basename,'_conversion_info.txt');

    % Factor to scale map.data to nm
    nm_scaling = map.scaling*1e9;
    
    % Output map and temporary map
    map_out=map;
  
    disp(' --- recentering map ...');
    % Recenter map
    map_out=FT_recenter_mirror_map(map_out);
    
    % Map radius in m
    real_radius=map.xstep*FT_find_map_radius(map);
  
    [y,x]=size(map.data);

    disp(' --- convolution between map and Zernike terms ...')
    % Convolution between map and up to Zernike order 2
    zc = FT_zernike_map_convolution(map_out,2,zc_name);
    
    disp(' --- removing Zernike terms Z00, Z11 and Z20 ...')
    disp(sprintf(' --- removing offset of A00 = %g [nm] ...',nm_scaling*zc.amp(1,1)))
    % Removing offset
    map_out = FT_remove_zernike_polynomial_from_map(map_out,0,0,zc.amp(1,1));
    
    disp(sprintf(' --- removing tilt of A11 = %g [nm] ...',nm_scaling*sqrt(zc.amp(2,1)^2+zc.amp(2,2)^2)))
    % Removing tilt -> -1 (odd polynomial), +1 (even polynomial)
    map_out = FT_remove_zernike_polynomial_from_map(map_out,1,-1,zc.amp(2,1));
    map_out = FT_remove_zernike_polynomial_from_map(map_out,1,1,zc.amp(2,2));
    
    if curvature<2
        % Remove curvature
        disp(sprintf(' --- removing curvature of A20 = %g [nm] ...',nm_scaling*zc.amp(3,2)))
        map_out = FT_remove_zernike_polynomial_from_map(map_out,2,0,zc.amp(3,2));
    end
    
    if curvature==1
        % Remove astigmatism
        disp(sprintf(' --- removing astigmatism of A22 = %g [nm] ...',nm_scaling*sqrt(zc.amp(3,1)^2+zc.amp(3,3)^2)))
        map_out = FT_remove_zernike_polynomial_from_map(map_out,2,-2,zc.amp(3,1));
        map_out = FT_remove_zernike_polynomial_from_map(map_out,2,2,zc.amp(3,3));
    end
    
    % Create aperture map
    a_map = FT_create_aperture_map(map_out);
    
    % Write maps in Finesse format
    disp(sprintf(' --- saving file "%s" ...',f_mapname))
    FT_write_surface_map(f_mapname,map_out)
    disp(sprintf(' --- saving file "%s" ...',a_mapname))
    FT_write_surface_map(a_mapname,a_map)
    
    % Write results to STDOUT
    write_results(1,map_out,curvature,zc);

    % ... and to file
    disp(sprintf(' --- writing results to file "%s" ...',results_name));
    FT_make_backup(results_name);
    [FID, result]=fopen(results_name,'w');  
    if (FID==-1)
        msgid=[baseid,':fileopen'];
        error(msgid,result);
    end
    write_results(FID,map_out,curvature,zc);
    fclose(FID);
  
    % and maybe plot the result
    FT_plot_mirror_map(map_out,0);

end


function []=write_results(FID,map,curvature,zc);
  
    % Calculate map radius [m] and equivalent optical features (angles for
    % tilt, Rcs for curvature etc.)
    R = map.xstep*FT_find_map_radius(map);
    OF1 = FT_zernike_optical_features(zc.amp(2,2),R,1,1);
    OF2 = FT_zernike_optical_features(zc.amp(2,1),R,1,-1);

    % use FID=1 for stdout
    % Output conversion data
    fprintf(FID,'\n---------------------------------------------------------\n');
    fprintf(FID,'Map: %s\n',map.name);
    fprintf(FID,'Date: %s\n', datestr(now,'dd. mmmm yyyy (HH:MM:SS)'));
    fprintf(FID,'---------------------------------------------------------\n');
    fprintf(FID,'Offset (Z00):         %3.3f nm\n',zc.amp(1,1)*map.scaling*1e9);
    fprintf(FID,'Tilt (Z11):           %3.3f nm\n',sqrt(zc.amp(2,1)^2+zc.amp(2,2)^2)*map.scaling*1e9);
    fprintf(FID,'                      (xbeta = %1.4g rad, ybeta = %1.4g rad)\n',OF1.value,OF2.value);
    if curvature<2
        OF3 = FT_zernike_optical_features(zc.amp(3,2),R,2,0);
        fprintf(FID,'Curvature (Z20):      %3.3f nm\n',zc.amp(3,2)*map.scaling*1e9);
        fprintf(FID,'                      (Rc = %3.3f m)\n',OF3.value);
    end
    if curvature==1
        fprintf(FID,'Astigmatism (Z22):    %3.3f nm\n',sqrt(zc.amp(3,1)^2+zc.amp(3,3)^2)*map.scaling*1e9);
    end
    fprintf(FID,'---------------------------------------------------------\n\n');

end

