%--------------------------------------------------------------------------
% function [result,dfile] =
% FT_FFT_cavity_zooms(cavity,field,Grid,lambda,accuracy,zooms,verbose)
%
% A function for Matlab which carries out an FFT simulation of a cavity,
% computing the power circulating inside the cavity as a function of the 
% cavity tuning (a cavity scan).  In this function multiple scans of the 
% cavity are performed, each one zooming in on the main resonance peak to 
% find the operating point of the cavity.
%
% cavity:   Structure storing the mirrors and spaces which make up the
%           cavity.
% field:    Array of complex numbers describing the field injected into the
%           cavity [sqrt(W)].
% Grid:     Structure describing grid used in FFT calculations (see
%           FT_init_grid.m).
% lambda:   Laser wavelength [m].
% accuracy: Accuracy of the intr cavity power (determines the number of
%           round-trips required).
% zooms:    Number of zooms to perform, i.e. 1 just returns the complete
%           cavity scan, over 1 lambda.
% verbose:  1:  plot each cavity scan
%
% result:   Structure storing final cavity scan (final zoom) and the
%           operating point.
% dfile:    Structure storing filenames and information about the files
%           containing precomputed round trip fields.
%
% Charlotte Bond    05/03/13
%--------------------------------------------------------------------------
%

function [result,dfile] = FT_FFT_cavity_zooms(cavity,field,Grid,lambda,accuracy,zooms,verbose)

    % Required round trips and number of points
    roundtrips = FT_FFT_required_cavity_roundtrips(cavity,accuracy);
    points = FT_FFT_required_scan_accuracy(cavity);
    
    % Precompute roundtrip fields
    dfile = FT_FFT_precompute_roundtrips(cavity,field,Grid,lambda,roundtrips,0);
    dfile = FT_FFT_reorganise_tmp_files(dfile,Grid);
    
    filename = sprintf('precompute_%s.mat',dfile.datestr);
    save(filename,'cavity','field','Grid','dfile')
    
    % !st zoom is full cavity scan
    fprintf(' ==== Full cavity scan ====\n');
    
    % Factor to zoom in by
    zoomfactor = 25/cavity.F;
    
    % Initial range
    range = lambda;
    result.operating_point = lambda/2;
    
    fprintf(' Estimated requirement: %d roundtrips, %d points for full scan.\n',roundtrips,points);
    
    % Perform scan for each zoom
    for i=1:zooms
        
        % Cavity scan
        [result.scan,scan_step] = FT_FFT_scan_range(result.operating_point,range,points);
        result.power = FT_FFT_cavity_scan(dfile,Grid,lambda,roundtrips,result.scan);
    
        % Save results of each scan
        save_xy = [result.scan' result.power'];
        if i==1
            save('full_scan.txt','-ASCII','save_xy')
        else
            savename = sprintf('zoom%g.txt',i-1)
            save(savename,'-ASCII','save_xy')
        end
    
        % Plot result
        if verbose==1
            figure
                semilogy(result.scan/lambda,result.power)
                title('cavity scan')
                xlabel('tuning [lambda]')
                ylabel('circulating power [W]')
                drawnow;
        end
        
        % Find operating point (point of maximum power)
        [result.max_power,result.max_scan_idx] = max(result.power);
        result.operating_point= result.scan(result.max_scan_idx);
        fprintf(' -> operating point %f [lambda] at max power %f [W] \n',result.operating_point/lambda, result.max_power); 
        
        % Calculate new scan range
        range = range*zoomfactor;
        FWHMrange = round(range/(lambda/cavity.F));
        
        if i~=zooms
            fprintf(' ==== Zooming in on maximum peak, range = %g FWHM ====\n',FWHMrange)
        end
        
    end
    
end

