%
%--------------------------------------------------------------------------
% function [R] = FT_find_map_radius(map,varargin)
% 
% A function for Matlab which finds the maximum radius from a defined
% optical centre of a set of data from a mirror map.  The data defined as
% inside the map surface (i.e. not nan) should be roughly circular/
% elliptical.  This function is designed to find the radius of the rough 
% circular surface.  The map should be centered to work correctly (see 
% FT_recenter_mirror_map.m).
%
% map:      map containing the surface height data of the optical component
%           we want to find the radius of.
% option:   Optional argument.  
%           'max':  (default) Computes the radius from the maximum 
%                   radii of  the valid pixels (for use with
%                   fitting functions).
%           'xy':   Computes radii in x and y directions.
%           'area': Computes radii by computing the area of the circle
%                   (number of valid pixels) and using R=sqrt(A/pi).  Only 
%                   works for properly circular data.  
%
% R:        Returned radius [indices].  For option 'xy' have R = [Rx Ry].
%
% Part of the Simtools package, http://www.gwoptics.org/simtools
% Charlotte Bond    03.01.10
%--------------------------------------------------------------------------
%

% Description: estimates the radius of a mirror given a mirror map
% Keywords: radius, mirror, map

function [R] = FT_find_map_radius(map,option)
  
    baseid='find_map_radius';
    
    % find the row and column indices which are on the component surface
    [ridx,cidx]=find(map.notnan==1);
    
    % define x and y coordinates to start at the optical centre of the map
    X = cidx-map.x0;
    Y = ridx-map.y0;
    
    if nargin==1 || strcmp(option,'max')
        % Find maximum radius
        r = sqrt(X.^2+Y.^2);
        R = max(r);
    elseif strcmp(option,'xy')
        % Find the radius in x/y
        R(1) = max(abs(X))+0.5;
        R(2) = max(abs(Y))+0.5;
    elseif strcmp(option,'area')
        % Use area to find radius
        A = numel(ridx);
        R = sqrt(A/pi);
    else
        % Error message
        result='Invalid option: must be "max", "xy" or "area"';
        msgid=[baseid,':checkarguments']
        error(msgid,result);
    end
                    
end

