%--------------------------------------------------------------------------
% function [k] = FT_map_coupling(map,gp,type,n,m,n_,m_)
%
% A function for Matlab which calculates the coupling on reflection between
% two gaussian modes when one is incident on a mirror described by a map.
%
% map:      Mirror surface map in standard structure (see
%           FT_new_surface_map.m)
% gp:       Gaussian beam parameter (see FT_init_gauss_param.m).
% type:     Type of gaussian mode.
%           'HG':       Hermite-Gauss
%           'LG':       Laguerre-Gauss
%           'LGcos':    sinusoidal Laguerre-Gauss
% n,m:      Incident mode indices
% n_,m_:    Coupled mode indices
%
% k:        Returned coupling coefficient
%
% Charlotte Bond    15.10.2013
%--------------------------------------------------------------------------
%

function [k] = FT_map_coupling(map,gp,type,n,m,n_,m_)
    
    % Get mirror surface distortion in m
    [x,y] = FT_get_map_axes(map);
    Z = map.data*map.scaling;
    
    % Calculate phase of reflected beam
    k = 2*pi/gp.lambda;
    Axy = exp(2i*k*Z);
    
    % Define fields of incident and coupled beam
    if strcmp(type,'HG')
        field_in = FT_HG_field(gp,n,m,x,y,[0,0]);
        field_out = FT_HG_field(gp,n_,m_,x,y,[0,0]);
    elseif strcmp(type,'LG')
        field_in = FT_LG_field(gp,n,m,x,y,[0,0,0]);
        field_out = FT_LG_field(gp,n_,m_,x,y,[0,0,0]);
    elseif strcmp(type,'LGcos')
        field_in = FT_LG_cos_field(gp,n,m,x,y,[0,0,0]);
        field_out = FT_LG_cos_field(gp,n_,m_,x,y,[0,0,0]);
    end
    
    [idx] = map.notnan;
    
    % Apply phase for reflected field
    field_refl = field_in.*Axy;
    
    % Calculate coupling from reflected field
    k = FT_conv_fields(field_out(idx),field_refl(idx));

end

