%--------------------------------------------------------------------------
% function [gc] = FT_decompose_finesse_beam(filename,gp,max_order)
%
% A function for Matlab which takes a Finesse output file which has been
% used to detect a carrier input light field (beam name 0 node), detecting
% with yaxis abs:deg (i.e. amplitude and phase) and plots the light field
% intensity and decomposes the field into HG (or LG) modes.
%
% filename:     Finesse .out file
% gp:           Gaussian beam parameter structure
% type:         'HG','LG' or 'LGcos'
% max_order:    The maximum order of the decomposition
%
% Charlotte Bond    15.10.2013
%--------------------------------------------------------------------------
%

function [gc] = FT_decompose_finesse_beam(filename,gp,type,max_order)

    % Load and reshape data
    data = load(filename);
    [rows,cols]=size(data);
    X=data(:,1);
    Y=data(:,2);
    Z=data(:,3:cols);
    [x,y,z]=convert3D(X,Y,Z);

    % Calibrate x and y axes
    x = x*gp.w0(1);
    y = y*gp.w0(2);
    
    % Calculate field
    field = z(:,:,1).*exp(1i*z(:,:,2)*pi/180);
    
    % Plot intensity of field
    figure
    surfc(x,y,abs(field).^2,'edgecolor','none')
    view(2)
    axis tight
    axis square
    xlabel('x [x/wx0]')
    ylabel('y [y/wy0]')
    colorbar;
    
    % Do mode decomposition
    gc = FT_init_gauss_coefficients('HG',max_order);
    gc = FT_mode_content(gc,gp,field,x,y,[0,0,0]);
    FT_print_gauss_coefficients(gc,1e-6,0);
    
end

%---------------------------------------------------------------
% Utility function to convert Finesse 3D data into Matlab format
function [x,y,M]=convert3D(X,Y,Z)
[row,col]=size(Z);
nxp=length(find (Y(:)==Y(1)));
nyp=row/nxp;
y=Y(1:nyp);
x=X(1:nyp:row);
M=zeros(nyp,nxp,col);
for i=1:col
M(:,:,i)=reshape(Z(:,i),nyp,nxp);
end
end


