%--------------------------------------------------------------------------
% function [q] =
% FT_get_finesse_beam_parameter(FT,katname,nodename,mirrorname,verbose)
%
% A function for Matlab which runs a FINESSE simulation to detect the beam
% parameter at a given node in a particular FINESSE setup.  This parameter
% refers to the basis at a given node.
%
% FT:           Structure containing FINESSE location and name.
% katname:      Name of input FINESSE file defining the interferometer
%               structure and without any detectors or x/y axis commands.
% nodename:     Name of node at which to detect the beam parameter.
% mirrorname:   Name of mirror to tune to produce FINESSE output.  Beam
%               parameter will not change with tuning, generally chose
%               mirror connected to node.
% verbose:      0:  no plot
%               1:  plot output as check
%
% q:            Returned complex q parameter.  qx = q(1), qy = q(2)
%
% Charlotte Bond    31.01.2013
%--------------------------------------------------------------------------
%

function [q] = FT_get_finesse_beam_parameter(FT,katname,nodename,mirrorname,verbose)
    
    % Copy blocks from the input file
    outbl = FT_read_blocks_from_file(katname);
    
    % Number of blocks in katname
    nb = length(outbl);
    
    % New katname
    katname = FT_remove_file_extension(katname);
    katname = sprintf('%s_detect_param.kat',katname);
    
    for i=1:2
        
        % Detect beam parameter in x and y direction
        if i==1
            xy = 'x';
        else
            xy = 'y';
        end
    
        % Create new block storing the commands to detect the beam parameters
        outbl(nb+1) = FT_create_new_block('beam_params');
    
        % Add beam parameter detector at given node
        outbl(nb+1) = FT_add_line_to_block(outbl(nb+1),sprintf('bp bp0 %s q %s',xy,nodename));

        % Default x/y-axis (bp shouldn't change with mirror tuning)
        outbl(nb+1) = FT_add_line_to_block(outbl(nb+1),sprintf('xaxis %s phi lin -1 1 100',mirrorname));
        outbl(nb+1) = FT_add_line_to_block(outbl(nb+1),'yaxis re:im');
        
        % Write katfile
        FT_write_blocks_into_file(outbl,katname);
    
        fprintf('Detecting beam parameter at node %s \n',nodename)
        % Write over original file (just adding commands for beam detection)
        Run = FT_create_new_kat_run();
        Run.filename = katname;
        Run = FT_run_kat_simulation(FT,Run,1);
        
        % Compute one value for complex beam parameter
        q(i) = Run.y(1,1) + 1i*Run.y(1,2);
        
        if verbose==1
            % Plot results
            figure
                hold on;
                % Plot real and imaginary parts
                plot(Run.x,Run.y(:,1),'LineWidth',2)
                plot(Run.x,Run.y(:,2),'r--','LineWidth',2)
                xlabel('tuning [deg]')
                ylabel('abs')
                legend('Re(q)','Im(q')
                title('Beam parameter')
                drawnow;
        end
        
    end        
    
    
end

