%--------------------------------------------------------------------------
% function [Run] =
% FT_ad_scan(FT,Run,blocks,mirror,node,f,nm,tuning,range,npoints)
%
% A function for Matlab which detects the amplitude of a light field in
% FINESSE (using an amplitude detector) for a given FINESSE setup while a
% given mirror is tuned.
%
% FT:       FINESSE structure storing the name and location of the FINESSE
%           executable.
% Run:      Structure containg information about the current FINESSE run,
%           including the name of the FINESSE file to write (see
%           FT_create_new_kat_run.m).
% blocks:   Structure containg blocks which store the lines from a FINESSE
%           file detailing the interferometer setup.  For this function the
%           blocks should not contain lines including detectors or axes.
% mirror:   Name of the mirror to be tuned.
% node:     Name of the node to detect the light field at.
% f:        Frequency of the light field to detect, as an offset from the
%           carrier.  For the carrier use f=0 [Hz].
% nm:       Hermite-Gauss mode indices.  The amplitude detector can be used
%           to detect the amplitude of a particular mode at a particular
%           frequency.  To detect the overall amplitude (no particular
%           mode) just use nm=0;
% tuning:   The original tuning of the mirror [deg].
% range:    The range of the scan.  The mirror is tuned from tuning-range
%           to tuning+range [deg].
% npoints:  Number of points in scan.
%
% Run:      Returned run structure containing output data from the FINESSE
%           simulation, i.e. x = Run.x, y = Run.y, where x is a vector of
%           tunings and y is a vector of amplitudes.
%
% Charlotte Bond    15.02.2013
%--------------------------------------------------------------------------
% 

function [Run] = FT_ad_scan(FT,Run,blocks,mirror,node,f,nm,tuning,range,npoints)

    % Tuning paramters
    xstart = tuning-range;
    xend = tuning+range;
    
    % ad name for different frequencies
    if f==0
        fname = 'carrier';
    else
        fname = 'f';
    end
    
    % ad name/params for case of HG mode detection or not
    if length(nm)==1
        nmname = '';
        nmstr = '';
    else
        nmname = sprintf('%g%g',nm(1),nm(2));
        nmstr = sprintf('%g %g',nm(1),nm(2));
    end
        
    % Text line for amplitude detector and tuning of mirror via xaxis
    adline = sprintf('ad ad%s%s %s %g %s',fname,nmname,nmstr,f,node);
    xline = sprintf('xaxis %s phi lin %6.16f %6.16f %d',mirror, xstart, xend, npoints);
    
    % Add command lines to new block
    block_ad = FT_create_new_block('ad_scan');
    block_ad = FT_add_line_to_block(block_ad,adline);
    block_ad = FT_add_line_to_block(block_ad,xline);
  
    % Write blocks into kat file and run FINESSE simulation
    FT_write_blocks_into_file([blocks,block_ad],Run.filename);
    Run = FT_run_kat_simulation(FT,Run);

end

