%---------------------------------------------------------------------------------------------------
% function [DOF] = FT_michelson_DOF_tuning(blocks,mirrors,node,type,op_point,tuning_params,verbose)
%
% A function for Matlab which performs a FINESSE simulation of a given 
% optical setup describing a michelson interferometer with Fabry-Perot arm 
% cavities and possible dual recycling.  The function performs a simulation
% in FINESSE which tunes a particular degree of freedom (DOF) of the 
% interferometer: CARM,DARM, MICH, PRCL or SRCL, to a specified operating 
% point and returns the DOF tuning.  In this simulation the carrier field 
% is detected at a given node and the DOF is tuned to the given operating 
% point, zooming into the operating point several times to increase the 
% accuracy. This function is intended for initial tuning, rather than exact
% locking points, of the interferometer.
%
% blocks:        Block structure storing lines from a FINESSE file
%                specifying the optical setup of a Michleson
%                interferometer.  For this function these blocks
%                should not include any lines defining x/y-axis
%                commands or defining any detectors (ad/pd/beam etc.).
% mirrors:       A vector of strings which contains the names
%                (matching those in the FINESSE lines stored in
%                blocks) of the mirrors to tune for different
%                degrees of freedom.  The mirrors must be ordered
%                correctly for different DOF types (see below):
%                CARM: 1st mirror should be x-arm end mirror, 2nd
%                      should be y-arm end mirror, i.e. mirrors =
%                      {'ETMX' 'ETMY'}.
%                DARM: mirrors = {'ETMX' 'ETMY'}
%                MICH: mirrors = {'ETMX' 'ETMY' 'ITMX' 'ITMY'}
% node:          Name of the node in the FINESSE setup at which to
%                detect the amplitude of the carrier field.  This
%                should be chosen carefully depending on which
%                degree of freedom is being tuned.  I.e. the node
%                at the anti-symetric port (AS) would be a good
%                choice for DARM.
% type:          String describing degree of freedom to tune.
%                'CARM': Common arm length.  
%                        CARM = (Lx+Ly)/2
%                'DARM': Differential arm length.
%                        DARM = (Lx-Ly)/2
%                'MICH': Small-michelson arm length.
%                        MICH = (lx-ly)/2
% op_point:      The operating point type to tune to.  Possible
%                options are: 
%                'max': Tune DOF to point at which carrier
%                       amplitude is at a maximum.
%                'min': Tune DOF to point where carrier is at a
%                       minimum.
% tuning_params: Structure storing parameters for tuning along the
%                x-axis.
%                tuning_params.phi0:    Starting tuning for DOF [deg]
%                tuning_params.range:   Range over which to tune DOF
%                                       [deg]
%                tuning_params.npoints: Number of points to use in
%                                       scan
%                tuning_params.rel_err: Relative error allowed
%                                       between successive zooms
%                tuning_params.zooms:   Number of zooms into
%                                       operating point
% verbose:       1: plot scans
%                0: don't plot output
%
% DOF:           Returned tuning of degree of freedom for given
%                operating point [deg]
%                CARM = (Lx+Ly)/2
%                DARM = (Lx-Ly)/2
%                MICH = (lx-ly)/2
%
% TO DO: add PRCL/ SRCL
% Charlotte Bond    17.05.2013
%--------------------------------------------------------------------------
%

function [DOF] = FT_michelson_DOF_tuning(blocks,mirrors,node,type,op_point,tuning_params,verbose)

    % Error messages: DOF and number of mirrors


    % Tuning parameters
    tuning = tuning_params.tuning;
    range = tuning_params.range;
    npoints = tuning_params.npoints;
    rel_err = tuning_params.rel_err;
    zooms = tuning_params.zooms;
    
    % Initilise Finesse
    FT = FT_init_Finesse();
    
    % Create a new 'run' variable, a structure to hold FINESSE related data
    Run = FT_create_new_kat_run();
    
    % Set temporary filename
    tmpfile=sprintf('%s_tuning.kat',type);
    Run.filename=tmpfile;
        
    % Add commands to block for CARM (i.e. etmy should follow etmx)
    n = length(blocks);
    blocks(n+1) = FT_create_new_block(type);
    if strcmp(type,'CARM')
        blocks(n+1) = FT_add_line_to_block(blocks(n+1),sprintf('put %s phi $x1',mirrors{2}));
    elseif strcmp(type,'MICH')
        blocks(n+1) = FT_add_line_to_block(blocks(n+1),sprintf('put %s phi $mx1',mirrors{2}));
        blocks(n+1) = FT_add_line_to_block(blocks(n+1),sprintf('put %s phi $x1',mirrors{3}));
        blocks(n+1) = FT_add_line_to_block(blocks(n+1),sprintf('put %s phi $mx1',mirrors{4}));
    elseif strcmp(type,'DARM')
        blocks(n+1) = FT_add_line_to_block(blocks(n+1),sprintf('put %s phi $mx1',mirrors{2}));
    end

    % Finesse variables
    f0 = 0;
    err = 10*rel_err;
    i = 0;

    % Scan over smaller and smaller range until the number of zooms is
    % completed and the difference from the previous tuning is smaller than
    % the allowed error
    while err>rel_err || i<=zooms
        i = i+1;
        
        % Detect carrier amplitude as mirror is tuned
        Run = FT_ad_scan(FT,Run,blocks,mirrors{1},node,f0,0,tuning,range,npoints);
        
        % Find the operating point
        if strcmp(op_point,'max')
            new_tuning = Run.x(max(find(Run.y==max(Run.y))));
        elseif strcmp(op_point,'min')
            new_tuning = Run.x(max(find(Run.y==min(Run.y))));
        end
        
        % Relaive difference from previous operating point
        err = 2*abs(tuning-new_tuning)/abs(tuning+new_tuning);
        if isnan(err)
            err = 10*rel_err;
        end
        
        % Reset tuning and narrow range
        tuning = new_tuning;
        range = 4*range/npoints;
        
        % Plot scan if required
        if verbose==1
            figure
            semilogy(Run.x,Run.y)
            xlabel(sprintf('%s',type))
            ylabel(sprintf('carrier amplitude (%s) [a.u.]',node))
            drawnow;
        end
    end
    
    DOF = tuning;
    
    % Clean the FINESSE run
    FT_kat_clean(Run);

end

