%--------------------------------------------------------------------------
% function [gc,max_order] = FT_read_TEM_from_blocks(blocks)
%
% A function for Matlab which reads the tem lines defining the mode content
% of the laser input beam from blocks containing FINESSE commands.  This
% mode content is returned in the structure gc (see
% FT_init_gauss_coefficients.m).
%
% blocks:   Block structure storing FINESSE code.
%
% gc:       Structure storing coefficients describing the mode content of
%           the laser beam found in blocks.
% max_order:The maximum order of the modes in the inupt beam
%
% Charlotte Bond    24.05.2013
%--------------------------------------------------------------------------
%

function [gc,max_order] = FT_read_TEM_from_blocks(blocks)

    baseid = 'read_TEM_from_blocks';
    
    % Find blocks where tems (or tem*s) are defined
    % First look for tem
    Gtype = 'HG';
    temtype = 'tem';
    block_idx = FT_find_element_in_all_blocks(blocks,1,'tem');
    
    % If tem isn't found look for tem*
    if length(block_idx)==0
        Gtype = 'LG';
        temtype = 'tem*';
        block_idx = FT_find_element_in_all_blocks(blocks,1,'tem*');

    end
    
    if length(block_idx)==0
        % If can't find tem commands all laser power(and amp) is in
        % HG00 mode
        Gtype = 'HG';
        max_order = 0;
        gc = FT_init_gauss_coefficients(Gtype,max_order);
        gc = FT_set_mode_coefficient(gc,1,0,0);
    else
        % Otherwise read each tem/tem* mode
        n=0;
        for i=1:length(block_idx)
            
            tmp_block = blocks(block_idx(i));
            line_idx = FT_find_element_in_active_block(tmp_block,1,temtype);
            
            for j=1:length(line_idx)
                
                tmp_line = tmp_block.txt_lines(line_idx(j));
                n=n+1;
                
                % Read mode indices and coefficient
                index1(n) = cell2mat(FT_parse_line(tmp_line,3));
                index2(n) = cell2mat(FT_parse_line(tmp_line,4));
                power = cell2mat(FT_parse_line(tmp_line,5));
                phase = cell2mat(FT_parse_line(tmp_line,6));
                c(n) = sqrt(power)*exp(-1i*phase*pi/180);
                
            end
            
        end
        
        % Calculate maximum order of specified tem modes
        if strcmp(Gtype,'HG')
            max_order = max(index1+index2);
        else
            max_order = max(2*index1+abs(index2));
        end
        % Create gc structure storing mode amplitudes
        gc = FT_init_gauss_coefficients(Gtype,max_order);
        for n=1:length(index1)
            gc = FT_set_mode_coefficient(gc,c(n),index1(n),index2(n));
        end
        
    
    end
    

end

