%--------------------------------------------------------------------------
% function [R,T,L,phi] =
% FT_read_mirror_parameters_from_blocks(blocks,mirror)
%
% A function for Matlab which finds a FINESSE mirror in blocks containing
% FINESSE command lines and reads and returns the parameters associated
% with that mirror.
%
% blocks:   Blocks containing FINESSE command lines.
% mirror:   Name of the mirror to find.
%
% R:        Returned reflection coefficient.
% T:        Returned transmission coefficient.
% L:        Returned loss coefficient.
% phi:      Returned mirror tuning [deg].
%
% Charlotte Bond    19.02.2013
%--------------------------------------------------------------------------
%

function [R,T,L,phi] = FT_read_mirror_parameters_from_blocks(blocks,mirror)

    % Locate mirror in blocks and get mirror type
    [block_idx,line_idx,mtype] = FT_find_mirror_in_blocks(blocks,mirror);
    
    % Find each parameter (3 are given when the mirror is defined in
    % FINESSE)
    for i=3:5
        % Read parameter at the relevant position in the mirror command
        % line
        tmp_param = FT_get_element_from_line(blocks(block_idx).txt_lines(line_idx),i);
        % If '$' is found in the parameter string the value of the
        % parameter is given by a constant
        k = strfind(tmp_param,'$');
        if length(k)>0
            % Read constant value
            tmp_param = FT_remove_phrase_from_string(tmp_param,'$');
            params(i-2) = FT_read_const_from_blocks(blocks,tmp_param);
        else
            % Store parameter value
            params(i-2) = FT_kat_str2num(tmp_param);
        end
    end
    
    % Convert from string to number, including checking for FINESSE units,
    % i.e. u for micro etc.
    phi = params(3);
    
    % Convert reflection, transmission and loss coefficients to numbers and
    % calculating the final parameter (depending on which mirror type, 0,
    % 1 or 2 is used
    if mtype==0
        R = params(1);
        T = params(2);
        L = 1-T-R;
    elseif mtype==1
        T = params(1);
        L = params(2);
        R = 1-T-L;
    elseif mtype==2
        R = params(1);
        L = params(2);
        T = 1-R-L;
    end
    

end

