%--------------------------------------------------------------------------
% function [blocks] =
% FT_replace_mirror_parameter_in_blocks(blocks,mirror,parameter,value)
%
% A function for Matlab which finds a particular FINESSE mirror in the
% command lines contained in given blocks and then finds a given parameter
% of that mirror, replacing the current value.  
%
% blocks:       Blocks containing FINESSE input lines.
% mirror:       Name of the mirror to find in blocks.
% parameter:    Name of the parameter to replace.  Four different options:
%               'R':    reflectivity
%               'T':    transmission
%               'L':    loss
%               'phi':  tuning
% value:        New value of the parameter to be replaced.  R, T and L
%               should be between 0 and 1, phi should be in degrees.  Also
%               note that R+T+L=1 for any mirror.
% 
% blocks:       Returned blocks with the mirror parameter updated to the
%               given value.
%
% Charlotte Bond    19.02.2013
%--------------------------------------------------------------------------
%

function [blocks] = FT_replace_mirror_parameter_in_blocks(blocks,mirror,parameter,value)
    
    baseid = 'replace_mirror_parameter_in_blocks';
    
    % Find mirror position, the block, line index and the type of mirror
    % (0, 1 or 2)
    [block_idx,line_idx,mtype] = FT_find_mirror_in_blocks(blocks,mirror);

    % m postfix in FINESSE depending on mirror type
    if mtype==0
        ms = '';
    else
        ms = num2str(mtype);
    end
    
    % Parameter number in mirror command depends on which parameter we look
    % for and what mirror type is used
    if strcmp(parameter,'phi')
        param_no = 3;
    elseif strcmp(parameter,'R')
        if mtype==0 || mtype==2
            param_no = 1;
        else
            param_no = -1;
        end
    elseif strcmp(parameter,'T')
        if mtype==0
            param_no = 2;
        elseif mtype==1
            param_no = 1;
        else
            param_no = -1;
        end
    elseif strcmp(parameter,'L')
        if mtype>0
            param_no = 2;
        else
            param_no = -1;
        end
    end
    
    % Error message
    % If the mirror definition (i.e. m mname R T....) does not include the
    % parameter to change produce an error as changing this parameter
    % results in changes of the others
    if param_no==-1
        result=sprintf('Parameter not found: "m%s %s" does not specify %s in definition. \nChanging this parameter changes other coefficents.',ms,mirror,parameter);
        msgid=[baseid,':checkarguments']
        error(msgid,result);
    end
    
    % Read in the parameter from the relevant line
    tmp_param = FT_get_element_from_line(blocks(block_idx).txt_lines(line_idx),param_no+2);
    
    % Check to see if the parameter read in as a string includes a '$',
    % this indicates the parameter is given by a constant declared
    % somewhere else
    k = strfind(tmp_param,'$');
    if k>0
        % If the parameter is defined by a constant replace the value of
        % the constant with the new value
        const_name = FT_remove_phrase_from_string(tmp_param,'$');
        blocks = FT_replace_const_in_blocks(blocks,const_name,value);
    else
        % If the parameter is given in the mirror command change the value
        % here
        blocks(block_idx).txt_lines(line_idx)={FT_replace_value_in_line(blocks(block_idx).txt_lines(line_idx),param_no+2,value)};
    end
    
    
end

