%--------------------------------------------------------------------------
% function [map_out] = FT_prepare_phase_map_for_finesse(map,curvature,w)
%
% A Matlab function which prepares a (phase) mirror map for use in Finesse.
% Any curvature, tilt or offset of the surface is removed as these
% properties are specified with the finesse .kat file.  The function saves 
% the resulting map in the file format for use in Finesse, as well as 
% creating and saving a map representing the aperture created by the map.
% A file containing information on the properties removed from the maps is
% also produced.
%
% This function allows the user to specify if a gaussian weighting should
% be used when removing these properties.  If a guassian weight is
% specified the function uses fitting functions using fminsearch to fit and
% remove the curvature and tilt of the surface, weighting the fitting to
% get the best fit for a particular gaussian radius.  If a gaussian
% weighting is not required the function uses an inner product with
% specific Zernike polynomials (Z20, Z1+/-1, Z00).
%
% map:          Input map to be prepared and converted for Finesse
% offset:       [x0 y0] offset to the map optical centre ([0,0] for map 
%               centre) [mm]
% w:            Beam-size for gaussian weighting for curvature etc. 
%               removal [m].  For no gaussian weighting use w = 0;
%
% map_out:      Returned map, after preparation for Finesse
% 
% Part of the Simtools package, http://www.gwoptics.org/simtools
% Charlotte Bond    04.05.2012
%--------------------------------------------------------------------------
%

function [map_out] = FT_prepare_phase_map_for_finesse(map,offset,w)

    baseid = 'prepare_phase_map_for_finesse';

    % Output filenames
    basename = map.name;
    % Finesse map file
    f_mapname = sprintf('%s',basename,'_for_finesse.txt');
    % Finesse aperture file
    a_mapname = sprintf('%s',basename,'_aperture.txt');
    % File storing results of preparation, fittings etc.
    results_name = sprintf('%s',basename,'_conversion_info.txt');

    % set display style for fitting routines to 'final'
    display_style=0;

    % Factor to scale map.data to nm (usually 1)
    nm_scaling = map.scaling*1e9;

    % Output map and temporary map
    map_out = map;
  
    disp(' --- recentering map ...');
    % Recenter map
    map_out = FT_recenter_mirror_map(map_out);
    
    % Remove invalid outside elements
    map_out = FT_remove_elements_outside_map(map_out);

    % Map radius
    R = map_out.xstep*FT_find_map_radius(map_out);
    
    disp('--- removing curvature ...')
    % Remove curvature using Zernike polynomials.  If gaussian weighting is
    % required a fitting function is used and this curvature (Rc_temp) is
    % used as the initial guess of the curvature
    [map2,A2,Rc_temp] = FT_remove_zernike_curvatures_from_map(map_out,'constant');
    if w~=0
        % If weighting required use fitting function
        [map_out,Rc,A0_1] = FT_remove_curvature_from_mirror_map(map_out,Rc_temp(1),w,display_style);
        % Equivalent Zernike amplitude
        A2 = FT_Rc_to_Znm(Rc,R);
        % Offset from fitting function.  -A2 is removed as this offset is
        % automatically part of the curvature polynomial (i.e. Z20 =
        % A20*(r^2 - 1).
        A0_1 = A0_1+A2;
    else
        map_out = map2;
        Rc = Rc_temp(1);
    end
    disp(sprintf(' --- removing curvature of A20 = %g [nm] ...',A2))
    disp(sprintf('                          (Rc  = %g [m] ...',Rc))

    disp('--- removing offset ...')
    % Remove average offset over beam-size/mirror
    if w~=0
        % If weighting function is used remove the average offset over the
        % beam radius
        if w<R
            [map_out,A0] = FT_remove_offset_from_mirror_map(map_out,w);
        else
            [map_out,A0] = FT_remove_offset_from_mirror_map(map_out,R);
        end
        % Put into nm
        A0 = A0*nm_scaling+A0_1;
    else
        [map_out,A0] = FT_remove_zernike_offset_from_map(map_out);
    end
    disp(sprintf(' --- removing offset of A00 = %g [nm] ...',A0))

    disp('--- removing piston ...')
    % Remove piston
    if w~=0
        % Use fitting function if gaussian weighting is required
        [map_out,xbeta,ybeta,A0_2] = FT_remove_piston_from_mirror_map(map_out,w,display_style);
        % Equivalent zernike amplitude
        A1 = R*tan([ybeta xbeta])*1e9;
        A0 = A0 + A0_2;
    else
        [map_out,A1,xbeta,ybeta] = FT_remove_zernike_tilt_from_map(map_out);
    end
    disp(sprintf(' --- removing tilt of A11 = %g [nm] ...',sqrt(A1(1)^2+A1(2)^2)))
    disp(sprintf('              (xbeta = %g [rad], ybeta = %g [rad])',xbeta,ybeta))
    
    % Offset map
    if offset(1)~=0 || offset(2)~=0
        % Add offset to mirror centre
        map_out.x0 = map_out.x0+offset(1)*1e-3/map_out.xstep;
        map_out.y0 = map_out.y0+offset(2)*1e-3/map_out.ystep;
    end
    
    % Create aperture map
    a_map = FT_create_aperture_map(map_out);
    
    % Write maps in Finesse format
    disp(sprintf(' --- saving file "%s" ...',f_mapname))
    FT_write_surface_map(f_mapname,map_out)
    disp(sprintf(' --- saving file "%s" ...',a_mapname))
    FT_write_surface_map(a_mapname,a_map)
    
    % Write results to terminal 
    write_results(1,map_out,A0,A1,A2,Rc,xbeta,ybeta,offset);

    % ... and to file
    disp(sprintf(' --- writing results to file "%s" ...',results_name));
    FT_make_backup(results_name);
    [FID, result]=fopen(results_name,'w');  
    if (FID==-1)
        msgid=[baseid,':fileopen'];
        error(msgid,result);
    end
    write_results(FID,map_out,A0,A1,A2,Rc,xbeta,ybeta,offset);
    fclose(FID);
  
    % and maybe plot the result
    FT_plot_mirror_map(map_out,0);

end


function []=write_results(FID,map,A0,A1,A2,Rc,xbeta,ybeta,offset);
  
    % Calculate map radius [m] 
    R = map.xstep*(FT_find_map_radius(map,'max'));
   
    % use FID=1 for stdout
    % Output conversion data
    fprintf(FID,'\n---------------------------------------------------------\n');
    fprintf(FID,'Map: %s\n',map.name);
    fprintf(FID,'Date: %s\n', datestr(now,'dd. mmmm yyyy (HH:MM:SS)'));
    fprintf(FID,'---------------------------------------------------------\n');
    fprintf(FID,'Diameter:             %3.3f cm\n',2*R*1e2);
    fprintf(FID,'Offset (Z00):         %3.1f nm\n',A0);
    fprintf(FID,'Tilt (Z11):           %3.1f nm\n',sqrt(A1(1)^2+A1(2)^2));
    fprintf(FID,'(xbeta = %2.2g urad, ybeta = %2.2g urad)\n',xbeta*1e6,ybeta*1e6);
    fprintf(FID,'Curvature (Z20):      %3.1f nm\n',A2);
    fprintf(FID,'(Rc = %3.1f m)\n',Rc(1));
    if offset(1)~=0 || offset(2)~=0
        fprintf(FID,'x0 offset:          %3.2f mm\n',offset(1));
        fprintf(FID,'y0 offset:          %3.2f mm\n',offset(2));
    end
    fprintf(FID,'---------------------------------------------------------\n\n');

end

