%--------------------------------------------------------------------------
% function [w1,w2,w0,z1,z2] = FT_cavity_beam_sizes(L,Rc1,Rc2,lambda)
%
% A function for Matlab which calculates the beam sizes and positions of
% the resonant beam in a linear cavity, using resonant g parameters.  The 
% beam sizes are calculated at each mirror of the cavity and at the waist.
%
% L:        Cavity length [m]
% Rc1/Rc2:  Radii of curvature of the input/end mirror [m].  Positive
%           numbers for those seen as concave from inside the cavity, 
%           negative for convex mirrors, as seen from inside the cavity.
%           For flat mirrors use Rc1/Rc2=0.  For astigmatic mirrors use
%           [Rc1x Rc1y]/[Rc2x Rc2y].
% lambda:   Laser wavelength [m]
%
% w1,w2,w0: Beam sizes at mirror 1, mirror 2 and at the waist [m].
% z1,z2:    Position of the mirror 1 and mirror 2 relative to the wast
%           position [m].  Negative values for before the waist, positive
%           for after.
%           For astigmatic cavities all w/z values will have an x and y
%           value, i.e. w1 = [w1x w1y].
%
% Charlotte Bond    08.02.2013
%--------------------------------------------------------------------------
%

function [w1,w2,w0,z1,z2] = FT_cavity_beam_sizes(L,Rc1,Rc2,lambda)

    baseid = 'cavity_beam_sizes';
    
    % Error messages
    % Checking number of Rc1/Rc2 values (should be 1 or 2)
    if (length(Rc1)>2 || length(Rc2)>2)
        result='Invalid Rc vectors: Rc1/Rc2 should have length 1 (non-astigmatic) \nor 2 (astigmatic)';
        msgid=[baseid,':checkarguments']
        error(msgid,result);
    end

    % Calculate individual mirror g-factors
    g1(1) = FT_g_factor(L,Rc1(1));
    g2(1) = FT_g_factor(L,Rc2(1));
    if length(Rc1)==2 || length(Rc2)==2
        g1(2) = FT_g_factor(L,Rc1(length(Rc1)));
        g2(2) = FT_g_factor(L,Rc2(length(Rc2)));
    end
    
    % Calculate beam sizes and positions from g-factors
    [w0,z1,zr,w1,w2] = FT_cavity_gg2w0z(g1,g2,L,lambda);

    % Calculate z2 and correct sign for z1
    z2 = L-z1;
    z1 = -z1;
    
end

