%--------------------------------------------------------------------------
% function [G] = FT_cavity_g_factor(L,Rc1,Rc2)
%
% A function for Matlab which calculates the g-factor for a cavity,
% G=g1*g2, where g1 and g2 are the g-factors for the individual mirrors.
% For a stable cavity 0<= G <=1.
%
% L:        Cavity length [m]
% Rc1/Rc2:  Radii of curvature of input and end mirrors [m].  Positive
%           numbers for those seen as concave from inside the cavity, 
%           negative for convex mirrors, as seen from inside the cavity.
%           For flat mirrors use Rc1/Rc2=0.  For astigmatic mirrors use
%           [Rc1x Rc1y]/[Rc2x Rc2y].
%
% G:        Returned cavity g-factor.  If the cavity is astigmatic reurned
%           g-factor is [Gx Gy].
%
% Charlotte Bond    08.02.2013
%--------------------------------------------------------------------------
%

function [G] = FT_cavity_g_factor(L,Rc1,Rc2)

    baseid = 'cavity_g_factor';
    
    % Error messages
    % Checking number of Rc1/Rc2 values (should be 1 or 2)
    if (length(Rc1)>2 || length(Rc2)>2)
        result='Invalid Rc vectors: Rc1/Rc2 should have length 1 (non-astigmatic) \nor 2 (astigmatic)';
        msgid=[baseid,':checkarguments']
        error(msgid,result);
    end

    % Calculate individual mirror g-factors
    g1(1) = FT_g_factor(L,Rc1(1));
    g2(1) = FT_g_factor(L,Rc2(1));
    if length(Rc1)==2 || length(Rc2)==2
        g1(2) = FT_g_factor(L,Rc1(length(Rc1)));
        g2(2) = FT_g_factor(L,Rc2(length(Rc2)));
    end
    
    % Cavity g-factor
    G = g1.*g2;

end

