%--------------------------------------------------------------------------
% function [xshift,yshift] = FT_match_cavity_scans(x,y,ymatch)
%
% A function for Matlab which takes two lots of data representing two
% different cavity scans and tries to match the data up by shifting one of
% the data sets.  This is simply achieved by shifting one set of data so
% that the point at which both scans are at a maximum matches up.  A useful
% function for comparing different scans.  The two scans should be over the
% same x-range (i.e. mirror tuning), which should be at least an FSR (or 
% multiples of).  It is preferable to use scans with an equal number of 
% points, otherwise the closest matching point is chosen.
%
% x:        x data for scan to be shifted (i.e. tuning data).  Should cover
%           an integer multiple of FSRs.
% y:        y data for scan to be shifted (i.e. power/amplitude)
%           coresponding to x data. 
% ymatch:   y data for the scan to match to (i.e. power/amplitude).
%
% xshift:   Returned shifted x data.
% yshift:   Returned shifted y data.
%
% Charlotte Bond    09.04.2013
%--------------------------------------------------------------------------
%

function [xshift,yshift] = FT_match_cavity_scans(x,y,ymatch)

    % Find the point at which ymatch is a maximum 
    imatch = find(ymatch==max(ymatch))-1;
    
    % Point at which y is a maximum
    i = find(y==max(y))-1;
    
    % Scale for converting between ymatch indices and y indices
    iscale = (length(y)-1)/(length(ymatch)-1)
    
    % Shift required to match y peak to ymatch peak
    shift = round(iscale*imatch(1)) - i(1);
    
    % Shift x and y data to match peaks up with ymatch
    [xshift,yshift] = FT_shift_cavity_scan(x,y,shift);

end

