%--------------------------------------------------------------------------
% function [xshift,yshift] = FT_shift_cavity_scan(x,y,shift)
% 
% A function for Matlab which shifts data describing a full cavity scan by
% a given number of data points.  The data should be a full scan over an
% FSR (or multiples of) as, depending on the shift sign, the data from the
% start of the scan is removed and stiched onto the end of the shifted
% data, or the data from the end is removed and stitched onto the start.  A
% useful function for use when comparing different cavity scans.
%
% x:        x data for cavity scan (i.e. mirror tuning).  Should cover 
%           n*FSR, where n is an integer.
% y:        y data for cavity scan (i.e. power/amplitude) coresponding to x
%           data.
% shift:    The number of points the data should be shifted by.  A positive
%           value results in the data being shifted to the right and data 
%           from the end of the original scan being moved to the begining 
%           of the shifted data.  The shifted x data will start at a lower 
%           value than the original.  A negative value will shift the scan 
%           data to the left and start at a higher x value.  The shift
%           should not be larger than the umber of points in the x/y data.
%
% xshift:   Returned x data, after shift.
% yshift:   Returned shifted y data. 
%
% Charlotte Bond    09.04.2013
%--------------------------------------------------------------------------
%

function [xshift,yshift] = FT_shift_cavity_scan(x,y,shift)

    baseid='shift_cavity_scan';

    % Error messgage if shift is greater than length of scan
    if(shift>length(y))
        msg='Invalid value: shift must not be larger than the length of x/y';
        msgid=[baseid,':checkarguments']
        error(msgid,result);
    end

    % Initial shifted data
    yshift = y;
    xshift = x;
    
    % x step size
    xstep = x(2)-x(1);
    
    % Length of scan data
    ilast = length(y);

    % Shift data depending on shift sign
    if shift<0
        % Shift y data
        yshift(1:ilast+shift) = y(1-shift:ilast);
        yshift(ilast+shift+1:ilast) = y(1:-shift);
        % Shift x data
        xshift = x(1-shift):xstep:x(1-shift)+(ilast-1)*xstep;
    elseif shift>0
        % Shift y data
        yshift(1+shift:ilast) = y(1:ilast-shift);
        yshift(1:shift) = y(ilast-shift+1:ilast);
        % Shift x data
        xshift = x(1)-shift*xstep:xstep:x(ilast-shift);
    end

end

