%--------------------------------------------------------------------------
%  function plate=FT_HG_phaseplate(nxpix,nypix,xsize,ysize,n,m,scale)
%  (requires herroots.m)
%  
%  Matlab function to fill a 2D grid with 1s and -1s, thus simulating a
%  spatial light phase modulator. Areas of the grid filled with 1s are
%  akin to purely transmissive regions of the plate, whereas areas filled 
%  with -1s are akin to areas providing a pi phaseshift. The grid is
%  designed such as to provide an optimised phaseplate pattern to convert
%  from a non-astigmatic TEM00 beam into any higher order HG beam. The
%  function automatically sets the phaseplate pattern to be optimised for
%  equal beam sizes in x and y-directions. If you wish to use square pixels
%  you should be careful that the ratios xsize:nxpix and ysize:nypix are
%  equal.
%
%  nxpix,nypix (int): Number of pixels along x and y-axes
%  n,m (int):         Mode indices of HG_nm mode to be generated
%  wx (real):         Desired beam radius in x-axis 
%  wy (real):         Desired beam radius in y-axis 
%
%  plate (real):      2D grid of 1s and -1s
%  x,y (real):        Vectors containing the x and y-positions of pixels
%  
%  Paul Fulda 17/05/09
%--------------------------------------------------------------------------

 function [plate,x,y]=FT_HG_phaseplate(nxpix,nypix,xsize,ysize,n,m,scale)

x=linspace(-xsize/2,xsize/2,nxpix);    % Generate vector of pixel x positions
y=linspace(-ysize/2,ysize/2,nypix);  % Generate vector of pixel y positions

% Generate a blank phaseplate of the required size
plate=ones(nypix,nxpix);

% Ensure the scaling is equal for both axes by defining the scale for the
% smaller axis

if nxpix>nypix
    scalepix=nypix;
else
    scalepix=nxpix;
end

if n>m
    rootscale=m;
else
    rootscale=n;
end

% Generate a vector containing the column numbers where flips should occur
if n==1
    xroots=nxpix/2+1;   % Avoid a 'divide by zero' error if n=1
else
xroots=round(nxpix/2+scalepix/2*scale*herroots(n)/max(herroots(rootscale)))+1; % Calculate the column numbers where flips occur
end

% Generate a vector containing the row numbers where flips should occur
if m==1
    yroots=nypix/2+1;   % Avoid a 'divide by zero' error if m=1
else
% Calculate the row numbers where flips occur
yroots=round(nypix/2+scalepix/2*scale*herroots(m)/max(herroots(rootscale)))+1; 
end

startx=1;  % Initialise the start variables
starty=1;

% Loop over the n+1 different sections along the x-axis, implementing flips
% at each required point

for j=1:n
   
   plate(:,startx:xroots(j))=(-1)^j;
   
   startx=xroots(j);
   
end

if n==0   % If statement to ignore next command if n=0
else
plate(:,startx:nxpix)=(-1)^(j+1);  % Flip the final section along the x-axis
end

% Loop over the m+1 different sections along the y-axis, implementing flips
% at each required point

for k=1:m
    
    plate(starty:yroots(k),:)=plate(starty:yroots(k),:)*((-1)^k);
    
    starty=yroots(k);
    
end

if m==0       % If statement to ignore next command if m=0
else
   plate(starty:nypix,:)=plate(starty:nypix,:)*((-1)^(k+1)); % Flip the final section along the y-axis
end

return