%--------------------------------------------------------------------------
%  function [plate]=FT_LGcosplate(lambda,xpoints,ypoints,xsize,ysize,p,l,w,xangle,yangle,contour)
%  (requires LaguerrePol.m)
%
%  Matlab function to fill a 2D grid with phase values between 0 and 2*pi,
%  thus simulating a spatial light phase modulator.
%
%  lambda (real): Wavelength of light used
%  xpoints, ypoints (int): number of x and y pixels
%  xsize, ysize (real): physical sizes of phaseplate in x and y 
%  p,l (int):         Mode indices of LG_pl mode to be generated
%  w (real):          Spot size of the LG_pl mode to be generated at the
%                     phaseplate [m]
%  xangle, yangle (real): blazing angles in x and y
%  contour (bool): 1 for contouring, 0 for non-contouring
%
%  plate (complex):  structure containing phaseplate phase values, along
%  with other grid parameters
%  
%
%  Paul Fulda 17/07/13
%--------------------------------------------------------------------------

function [phaseplate]=FT_LGcosplate(lambda,xpoints,ypoints,xsize,ysize,p,l,w,xangle,yangle,contour)

% Generate a blank phaseplate of the required size, and initialise matrices
% that will hold the scaled radial, and azimuthal coordinates for the grid
mygrid=FT_init_grid(xpoints,ypoints,xsize,ysize,0,0);

plate=ones(mygrid.xpoints,mygrid.ypoints);

X=mygrid.X;
Y=mygrid.Y;

sr=sqrt(2)*sqrt(X.^2+Y.^2)/w; % Calculate radial coordinates and scale to beam size
phi=atan2(Y,X);   % Calculate azimuthal coordinates of each pixel

% Introduce azimuthal phase dependence
plate=plate.*sign(cos(l*phi));

% Introduce radial phase discontinuities
flip=sign(FT_LaguerrePol(p,l,sr.^2));
flip=(flip+1)*pi/2;
plate=plate+flip;
phaseplate.phase=plate;
phaseplate.phase=mod(phaseplate.phase,2*pi);
phaseplate.xaxis=mygrid.xaxis;
phaseplate.yaxis=mygrid.yaxis;
phaseplate.X=mygrid.X;
phaseplate.Y=mygrid.Y;

blazephase=mygrid.X*0;
blazephase=blazephase+mygrid.X*tan(xangle)*2*pi/lambda;
blazephase=blazephase+mygrid.Y*tan(yangle)*2*pi/lambda;
phaseplate.phase=phaseplate.phase+blazephase;
phaseplate.phase=mod(phaseplate.phase,2*pi);

% if required, contour the amplitude of the phaseplate
if contour==1
    gp=FT_init_gauss_param(lambda,1,'z',0,'w0',w);
    contouramp=abs(FT_LG_cos_field(gp,p,l,mygrid.xaxis,mygrid.yaxis,[0,0,0]));
    contouramp=contouramp/max(max(contouramp));
    phaseplate.phase=phaseplate.phase.*contouramp;
end





