%--------------------------------------------------------------------------
% function [c,p] =
% FT_approximate_paraboloid_for_thermal_distortion(beam,mirror,num_terms)
%
% A function for Matlab which calculates the approximate paraboloid for the
% thermal distortion of a mirror due to an incident beam absorbed in the
% coating or substrate.
%
% N.B. currently only valid for and LG00 incident beam.
%
% coeffs:   Coefficients for bessel expansion of thermal distortion.
% SV:       Saint-Venant correction.
% beam:     Structure storing beam parameters (FT_init_thermal_beam.m).
% mirror:   Structure storing mirror parameters (FT_init_thermal_mirror.m).
%
% c,p:      Parameters for best fitted paraboloid:
%           Z = c*r^2+p
%
% Part of the Simtools package, http://www.gwoptics.org/simtools
% Charlotte Bond    13.02.2012
%--------------------------------------------------------------------------
%

function [c,p] = FT_approximate_paraboloid_for_thermal_distortion(coeffs,SV,beam,mirror)

    X = mirror.X;
    a = mirror.a;
    w = beam.w;
    pois = mirror.pois;
    Y = mirror.Y;

    % Number of terms in expansion
    num_terms = length(coeffs);
    
    % Calculating bessel zeros
    zeta_0 = FT_bessel_zeros(0,X,num_terms);
    y0 = zeta_0.^2*w^2/(8*a^2);
    
    c0 = -2/w^2*exp(-y0).*y0;
    
    % Initial paraboloid parameters
    c = 0;
    p = 0;
    
    % Calculate parabaloid terms
    for s=1:num_terms
        
        c = c - c0(s)*coeffs(s);
        p = p + (1 - exp(-y0(s)))*coeffs(s); 
        
    end
    
    c = c + SV;
    p = p + SV*w^2/2 - c*w^2/2;

end

