%--------------------------------------------------------------------------
% function [c,p] =
% FT_approximate_paraboloid_for_thermal_lens(coeffs,X,a,w)
%
% A function for Matlab which computes the approximate paraboloid for a 
% thermal lens caused by heating from a laser beam absorbed in the coating
% or substrate.
%
% Currently only valid for LG00 incident beam.
%
% coeffs:   Coefficients for expansion of thermal lens in terms of bessel
%           functions.
% X:        Reduced radiation constant.
% a:        Mirror radius [m].
% w:        Beam radius [m].
% 
% c,p:      Parameters for best fitted parabaloid:
%           Z = c*r.^2 +p
%
% Part of the Simtools package, http://www.gwoptics.org/simtools
% Charlotte Bond    09.02.2012
%--------------------------------------------------------------------------
%

function [c,p] = FT_approximate_paraboloid_for_thermal_lens(coeffs,X,a,w)

    % Number of terms in expansion
    num_terms = length(coeffs);
    
    % Calculating bessel zeros
    zeta_0 = FT_bessel_zeros(0,X,num_terms);
    y0 = zeta_0.^2*w^2/(8*a^2);
    
    c0 = -2/w^2*exp(-y0).*y0;
    
    % Initial coefficients
    c = 0;
    p = 0;
    
    % Calculating approximate parabaloid parameters
    for s=1:num_terms
        
        c = c + c0(s)*coeffs(s);
        p = p + exp(-y0(s))*coeffs(s);
    end

    p = p - c*w^2/2;
   

end

