%--------------------------------------------------------------------------
% function [A] = FT_Rc_to_Znm(Rc,R)
%
% A function for Matlab which converts from radii of curvature values for a
% surface to the equivalent amplitude of the Zernike order 2 polynomials 
% Z20 (describing power, or curvature) and Z22 (the Zernike polynomial
% describing astigmatism).  In this case, where we use Rcx and Rcy, the
% correct Zernike polynomial for comparision is the even Zernike polynomial
% (Z2+2).
%
% Rc:       Radius of curvature [m].  For a symmetrically curved surface 
%           just one value for Rc is required.  For an astigmatic mirror
%           values for Rcx and Rcy should be given -> Rc = [Rcx Rcy].  For 
%           a flat surface use Rc/Rcx/Rcy=0.
% R:        Radius of surface aperture/ Zernike disc [m]
% 
% A:        Returned vector of Zernike order 2 amplitudes [nm].
%           A(1) = A20  (correspnding to amplitude of Z20, constant 
%                       curvature polynomial)
%           A(2) = A22  (amplitude of Z22, astigmatic polynomial)
%
% Charlotte Bond    26.03.2013
%--------------------------------------------------------------------------
%

function [A] = FT_Rc_to_Znm(Rc,R)

    baseid = 'Rc_to_Znm';
    
    % Error messages
    % Checking number of Rc values (should be 1 or 2)
    if length(Rc)>2
        result='Invalid Rc vector: Rc should have length 1 (non-astigmatic) \nor 2 (astigmatic)';
        msgid=[baseid,':checkarguments']
        error(msgid,result);
    end

    % Index of last Rc value
    idx = length(Rc); 
    Ax = 0;
    Ay = 0;
    
    % If curvature is non-zero calculate amplitude in x and y directions
    if Rc(1)~=0
        Ax = (1/2) * (Rc(1)-sign(Rc(1))*sqrt(Rc(1)^2-R^2));
    end
    
    if Rc(idx)~=0
        Ay = (1/2) * (Rc(idx)-sign(Rc(idx))*sqrt(Rc(idx)^2-R^2));
    end
    
    % Calculate Z20 amplitude
    A(1) = (Ax+Ay)/2;
    
    if length(Rc)>1
        % Calculate Z22 astigmatic amplitude
        A(2) = 2*(Ax-A(1));
    end

    % Convert amplitudes to nm
    A = 1e9*A;

end

