%--------------------------------------------------------------------------
% function [Rc] = FT_Znm_to_Rc(A,R)
%
% A function for Matlab which converts from Zernike curvature amplitudes to
% radii of curvature (maximum and minimum).  The Zernike curvature 
% amplitudes refer to the amplitude of Z20 (the curvature polynomial) and 
% Z22 (the astigmatic polynomial).  For the case of all the astigmatism 
% described by the even Zernike polynomial (Z2+2) the returned radii of 
% curvature refer to Rcx and Rcy.  Otherwise the refer to the minimum and
% maximum radii of curvature.
% 
% A:    Vector of Zernike amplitudes [nm].  For the case of a
%       non-astigmatic surface give just one value for A, refering to A20.  
%       For an astigmatic surface give [A20 A22], where A22 refers to the
%       amplitude of the astigmatic polynomial.
% R:    Radius of the surface aperture or Zernike disc [m].
%
% Rc:   Returned vector of radii of curvature, refering to [Rcx Rcy] [m].
%
% Charlotte Bond    26.03.2013
%--------------------------------------------------------------------------
%

function [Rc] = FT_Znm_to_Rc(A,R)

    baseid = 'Znm_to_Rc';
    
    % Error messages
    % Checking number of A values (should be 1 or 2)
    if length(A)>2
        result='Invalid A vector: A should have length 1 (non-astigmatic) \nor 2 (astigmatic)';
        msgid=[baseid,':checkarguments']
        error(msgid,result);
    end

    % Convert Zernike amplitudes to m
    A = 1e-9*A;
    
    % Select Z20 amplitude
    A20 = A(1);
    
    % Select Z22 amplitude
    if length(A)==2
        A22 = A(2);
    else
        A22 = 0;
    end

    % Calculate maximum and minimum amplitudes
    s = [+1 -1];
    Rc = ((2*A20 + s*A22).^2 + R^2)./(2*(2*A20+s*A22));

end

