%--------------------------------------------------------------------------
% function [fake_map] =
% FT_create_synthetic_zernike_map(zc,option,real_size,grid_size,R,mapname)
%
% A function for Matlab which computes a 'synthetic', or 'fake', map from a
% sum of Zernike polynomials.  The amplitudes of the Zernike polynomials
% are provided and the function can either create a surface map with these
% original amplitudes or randomly distribute the combined amplitude for all 
% the polynomials of a given order (n) between the polynomials of that 
% order.  The aim of this is to produce maps with similar spectra, but with 
% differing Zernike distributions.
%
% zc:           Structure storing the original amplitudes of the Zernike
%               polynomials, in nm.
% option:       'original': Use original amplitudes to recreate zernike
%                           map
%               'random':   Random distribution of amplitudes throughout 
%                           the orders, but the the sum of amplitudes in 
%                           each order remains the same. 
% realsize:     Size of the map [m]
% gridsize:     Size of the grid representing the map
% R:            Radius of mirror map [m].  For different values along x and
%               y axes use [Rx Ry]. 
% mapname:      Name of map
%
% fake_map:     Returned synthetic map (data in nm)
%
% Part of the Simtools package, http://www.gwoptics.org/simtools
% Charlotte Bond    12.04.2011
%--------------------------------------------------------------------------
%

function [fake_map] = FT_create_synthetic_zernike_map(zc,option,realsize,gridsize,R,mapname)

    baseid = 'create_synthetic_zernike_map';
    
    % Option should be 'random' or 'original'
    if (~strcmp(option,'original') && ~strcmp(option,'random'))
        result='Invalid option: option must be "original" or "random"';
        msgid=[baseid,':checkarguments']
        error(msgid,result);
    end
    
    [N,I]=size(zc.amp);
    nmax=N-1;
    
    % If option is 'random' then distribute amplitudes randomly throughout
    % zernike polynomials
    if strcmp(option,'original')
        fake_zc=zc;
    elseif strcmp(option,'random')
       
        temp_amps=zeros(N,I);
  
        % The sum of amplitudes in the same order should stay the same but is
        % randomly distributed over those with different m.
        for n=0:nmax
            A=sum(abs(zc.amp(n+1,1:n+1)));
            i=1;
            for m=-n:2:n
                temp_amps(n+1,i)=(2*rand()-1)*A;
                fake_zc.n(n+1,i)=n;
                fake_zc.m(n+1,i)=m;
                i=i+1;
          
            end
            temp_A=sum(abs(temp_amps(n+1,1:I)));
            fake_zc.amp(n+1,1:I)=(A/temp_A)*temp_amps(n+1,1:I);
            sum(abs(fake_zc.amp(n+1,1:I)));
      
        end
    end
  
    % Basic parameters for Zernike map
    rows=gridsize;
    cols=gridsize;
    stepsize=realsize/gridsize;
    x0=(cols+1)/2;
    y0=(rows+1)/2;
    rx = R(1)/stepsize;
    ry = R(end)/stepsize;
    
    data=0;
    
    % Calculate each zernike polynomial with the respective amplitude
    for n=0:nmax
        i=1;
        for m=-n:2:n
            Z=fake_zc.amp(n+1,i)*FT_zernike_polynomial_for_grid(cols,rows,x0,y0,rx,ry,n,m);
            % Add all the surface heights specified by the polynomials
            data=data+Z;
            i=i+1;
        end
    end

    % Create new map and add Zernike polynomials to surface (surface heights
    % are in nm)
    fake_map=FT_new_surface_map(mapname,data,0,0,x0,y0,stepsize,stepsize,1e-9);
  
    % Recenter map
    fake_map=FT_recenter_mirror_map(fake_map);
 
end

